package nl.voidcorp.bankplugin

import org.bukkit.Bukkit
import org.bukkit.entity.Player
import org.bukkit.event.EventHandler
import org.bukkit.event.Listener
import org.bukkit.event.inventory.InventoryClickEvent
import org.bukkit.event.inventory.InventoryCloseEvent
import org.bukkit.inventory.Inventory
import org.bukkit.inventory.ItemStack
import java.util.*


class IconMenu(
    private val name: String = "Buy/Sell",
    size: Int,
    private val click: (clicker: Player, menu: IconMenu, rowNum: Int, slot: Int, item: ItemStack?, event: InventoryClickEvent) -> Boolean
) : Listener {
    private val size: Int = size * 9
    private val viewing: MutableList<UUID> = mutableListOf()

    private val items: MutableList<ItemStack?> = MutableList(this.size) { null }

    fun open(p: Player): IconMenu {
        p.openInventory(getInventory(p))
        viewing.add(p.uniqueId)
        return this
    }

    private fun getInventory(p: Player): Inventory {
        val inv = Bukkit.createInventory(p, size, name)
        for (i in items.indices)
            inv.setItem(i, items[i])
        return inv
    }

    fun close(p: Player): IconMenu {
        viewing.remove(p.uniqueId)
        return this
    }

    @EventHandler
    fun onInventoryClick(event: InventoryClickEvent) {
        val p = event.whoClicked as Player
        if (p.uniqueId in viewing) {
            if (event.clickedInventory == event.whoClicked.inventory) {
                if (event.isShiftClick) {
                    event.isCancelled = true
                }

            } else {
                event.isCancelled = true
                val row = getRowFromSlot(event.slot)
                if (!click(p, this, row, event.slot % 9, event.currentItem, event))
                    close(p)
            }
        }
    }

    @EventHandler
    fun onInventoryClose(event: InventoryCloseEvent) {
        if (viewing.contains(event.player.uniqueId))
            viewing.remove(event.player.uniqueId)
    }

    fun addButton(
        row: Int,
        position: Int,
        item: ItemStack,
        name: String = "",
        vararg lore: String = arrayOf()
    ): IconMenu {
        items[row * 9 + position] = getItem(item, name, *lore)
        return this
    }

    fun getRowFromSlot(slot: Int): Int {
        return slot / 9
    }

    fun getRow(row: Int): MutableList<ItemStack?> {
        return items.subList(row * 9, row * 9 + 9)
    }

    private fun getItem(item: ItemStack, name: String, vararg lore: String): ItemStack {
        val im = item.itemMeta
        if (name.isNotBlank())
            im.setDisplayName(name)
        if (lore.isNotEmpty())
            im.lore = listOf(*lore)
        item.itemMeta = im
        return item
    }

}