package nl.voidcorp.bankplugin.commands

import nl.voidcorp.bankplugin.models.Bank
import nl.voidcorp.bankplugin.models.Banks
import nl.voidcorp.mainplugin.commands.VoidCommand
import org.bukkit.Bukkit
import org.bukkit.command.Command
import org.bukkit.command.CommandSender
import org.bukkit.command.ConsoleCommandSender
import org.bukkit.entity.Player
import org.jetbrains.exposed.sql.transactions.transaction
import kotlin.math.absoluteValue

class TransferCashCommand : VoidCommand() {
    override fun onTabComplete(
        sender: CommandSender,
        command: Command,
        alias: String,
        args: Array<out String>
    ): List<String> {
        return if (args.size <= 1)
            sender.server.onlinePlayers.filter { it.name.startsWith(args.getOrNull(0) ?: "") }.map { it.name }
        else emptyList()
    }


    override fun onCommand(sender: CommandSender, command: Command, label: String, args: Array<out String>): Boolean {
        return if (args.size == 2) {
            val to = Bukkit.getPlayer(args[0])
            val amt = args[1].toLongOrNull()
            if (amt == null) {
                sender.sendMessage("The amount needs to be a number, not '${args[1]}'...")
                true
            } else
                if (to != null && amt > 0) {
                    if (sender is Player) {
                        transaction {
                            val from = Bank.find { Banks.player eq sender.uniqueId }.firstOrNull() ?: Bank.new {
                                player = sender.uniqueId
                            }
                            val bank = Bank.find { Banks.player eq to.uniqueId }.firstOrNull() ?: Bank.new {
                                player = to.uniqueId
                            }
                            if (from.money >= amt) {
                                from.money -= amt
                                bank.money += amt
                                sender.sendMessage("You sent $amt to ${to.name}")
                                to.sendMessage("You received $amt from ${sender.name}!")
                            } else {
                                val missing = (from.money - amt).absoluteValue
                                sender.sendMessage("You are missing $missing ECs to send them to ${to.name}")
                            }

                        }
                    } else if (sender is ConsoleCommandSender) {
                        transaction {
                            val bank = Bank.find { Banks.player eq to.uniqueId }.firstOrNull() ?: Bank.new {
                                player = to.uniqueId
                            }
                            bank.money += amt
                            to.sendMessage("Hmm... you seem to have magically received $amt from the gods?")
                        }
                    }
                    true
                } else {
                    false
                }
        } else {
            false
        }
    }
}