package nl.voidcorp.bankplugin.costcalc

import nl.voidcorp.bankplugin.VoidBank
import nl.voidcorp.mainplugin.adapter
import nl.voidcorp.mainplugin.moshi
import org.bukkit.Bukkit
import org.bukkit.Material
import org.bukkit.Tag
import org.bukkit.inventory.FurnaceRecipe
import org.bukkit.inventory.ShapedRecipe
import org.bukkit.inventory.ShapelessRecipe
import kotlin.math.max

object Collector {
    private val prices = mutableMapOf(Material.IRON_INGOT to 256)

    init {
        reloadConfig()
    }

    fun reloadConfig() {
        val config = VoidBank.config.bankMap
        if (config.isNotEmpty() && !VoidBank.config.rerun) {
            prices.clear()
            prices.putAll(config.map { Material.matchMaterial(it.key)!! to it.value }.toMap())
        }
        if (VoidBank.config.rerun) {
            prices.putAll(Tag.LOGS.values.map { it to 32 })
            prices.putAll(Tag.PLANKS.values.map { it to 8 })
            prices.putAll(Tag.SAPLINGS.values.map { it to 32 })
            prices.putAll(Tag.LEAVES.values.map { it to 1 })
            prices.putAll(Tag.WOOL.values.map { it to 48 })
            this += Material.DIAMOND to 8192
            this += Material.STICK to 4
            this += Material.REDSTONE to 64
            this += Material.GLOWSTONE_DUST to 384
            this += Material.GOLD_INGOT to prices[Material.IRON_INGOT]!! * 8
            val borrow = moshi.adapter<ProjectEBorrow>()
                .fromJson(Collector::class.java.getResourceAsStream("/default.json").bufferedReader().readText())
            if (borrow != null) {
                prices.putAll(borrow.values.asMaterialList)
            }

            // let the pain commence
            val simple = mutableListOf<SimpleRecipe>()
            val recipes = Bukkit.recipeIterator().asSequence().filter { it.result.type !in prices.keys }.toMutableList()
            val shapeless = recipes.filterIsInstance<ShapelessRecipe>()
            val shaped = recipes.filterIsInstance<ShapedRecipe>()
            val furnace = recipes.filterIsInstance<FurnaceRecipe>()
            for (r in shapeless) {

                simple += SimpleRecipe(
                    r.ingredientList.groupingBy { it.type }.eachCount(),
                    r.result.type,
                    r.result.amount
                )
            }
            for (r in shaped) {
                val ingredients = r.ingredientMap
                val shape = r.shape.joinToString("").toList()
                val m = mutableMapOf<Material, Int>()
                for (c in shape) {
                    val i = ingredients[c]
                    if (i != null) {
                        val curr = m.getOrDefault(i.type, 0) + 1
                        m[i.type] = curr
                    }
                }
                simple += SimpleRecipe(m, r.result.type, r.result.amount)
            }
            for (r in furnace) {
                val sticks = r.cookingTime / 100
                val inp = r.input
                val outp = r.result
                val m = mutableMapOf(inp.type to inp.amount, Material.STICK to sticks)
                simple += SimpleRecipe(m, outp.type, outp.amount)
            }

            simple += SimpleRecipe(
                mapOf(Material.BUCKET to 1),
                Material.MILK_BUCKET,
                1
            )
            simple += SimpleRecipe(
                mapOf(Material.GOLD_BLOCK to 8, Material.APPLE to 1),
                Material.ENCHANTED_GOLDEN_APPLE,
                1
            )
            simple += SimpleRecipe(
                mapOf(Material.IRON_INGOT to 4),
                Material.CREEPER_HEAD,
                1
            )
            simple += SimpleRecipe(
                mapOf(Material.WITHER_SKELETON_SKULL to 4),
                Material.CREEPER_HEAD,
                1
            )
            simple += SimpleRecipe(
                mapOf(Material.COBBLESTONE to 16384 / 12),
                Material.MELON_SLICE,
                9
            )
            simple += SimpleRecipe(
                mapOf(Material.PUMPKIN to 1),
                Material.CARVED_PUMPKIN,
                1
            )
            simple += SimpleRecipe(
                mapOf(Material.COBBLESTONE to 2),
                Material.BAMBOO,
                1
            )

            repeat(5) {
                simple.removeIf { (it.output in prices.keys) or (it.input.keys.any { it.name.contains("ORE") }) }
                VoidBank.logger.info("Count: ${simple.size}, step $it")
                val rs = simple.filter { it.input.keys.all { it in prices.keys } }
                for (r in rs) {
                    val cost = r.input.map { it.value * prices.getOrDefault(it.key, 0) }.sum() / r.count
                    prices[r.output] = max(cost, prices.getOrDefault(r.output, 0))
                }
            }
            simple.removeIf { (it.output in prices.keys) or (it.input.keys.any { it.name.contains("ORE") }) }
            VoidBank.logger.info("Count: ${simple.size}, step end")
//        VoidBank.logger.info(moshi.adapter(meme::class.java).toJson(meme(simple)))
            prices.remove(Material.ENCHANTED_BOOK)
            prices.remove(Material.FILLED_MAP)
            prices.remove(Material.WRITTEN_BOOK)
            VoidBank.config.bankMap = prices.mapKeys { "minecraft:${it.key.name.toLowerCase()}" }.toMutableMap()
            VoidBank.config.rerun = false

        }
    }

    operator fun plusAssign(p: Pair<Material, Int>) {
        prices += p
    }

    operator fun get(m: Material) = prices.getOrDefault(m, 0)

    operator fun rangeTo(max: Int) = prices.filterValues { it <= max }

    data class SimpleRecipe(val input: Map<Material, Int>, val output: Material, val count: Int)
    class meme(val recipes: List<SimpleRecipe>)
}